/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * export - gtk handlers & code for exporting a plot as an image
 */

#include <gtk/gtkmessagedialog.h>
#include <gtk/gtkradiobutton.h>
#include <gtk/gtkfilechooser.h>
#include <gtk/gtkfontbutton.h>
#include <gtk/gtkcombobox.h>
#include <gtk/gtkversion.h>
#include <gtk/gtkdialog.h>

#include "export.xh"
#include "gtkutil.xh"
#include "compat.xh"
#include "paintapi-gtk.xh"

static gboolean export_image(my_plot_t *plot, const char *filename, const char *format, int width, int height, int backround_white, char *font);

typedef struct {
      GtkEntry *entry_reso_x;
      GtkEntry *entry_reso_y;
      GtkWidget *label_reso_times;

      GtkRadioButton *radiobutton_reso_plot;

      my_plot_t *plot;

} eac_callback_data_t;

static void eac_reso(eac_callback_data_t *data, const char *w, const char *h) {
   gtk_entry_set_text(data->entry_reso_x, w);
   gtk_entry_set_text(data->entry_reso_y, h);
   gtk_widget_set_sensitive(GTK_WIDGET(data->entry_reso_x), 0);
   gtk_widget_set_sensitive(GTK_WIDGET(data->entry_reso_y), 0);
   gtk_widget_set_sensitive(GTK_WIDGET(data->label_reso_times), 0);
}

static void eac_reso_640x480(GtkWidget *button, eac_callback_data_t *data) {
   eac_reso(data, "640", "480");
}

static void eac_reso_800x600(GtkWidget *button, eac_callback_data_t *data) {
   eac_reso(data, "800", "600");
}

static void eac_reso_1024x768(GtkWidget *button, eac_callback_data_t *data) {
   eac_reso(data, "1024", "768");
}

static void eac_reso_1280x1024(GtkWidget *button, eac_callback_data_t *data) {
   eac_reso(data, "1280", "1024");
}

static void eac_reso_1600x1200(GtkWidget *button, eac_callback_data_t *data) {
   eac_reso(data, "1600", "1200");
}

static void eac_reso_plot(GtkWidget *button, eac_callback_data_t *data) {
   char w[6], h[6];
   snprintf(w, sizeof(w), "%d", data->plot->da->allocation.width);
   snprintf(h, sizeof(h), "%d", data->plot->da->allocation.height);
   eac_reso(data, w, h);
}

static gboolean eac_plot_configure_event(GtkWidget *widget, GdkEventConfigure *do_not_use, eac_callback_data_t *data) {
   if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->radiobutton_reso_plot))) {
      eac_reso_plot(NULL, data);
   }
   return 0;
}

static void eac_reso_custom(GtkWidget *button, eac_callback_data_t *data) {
   gtk_widget_set_sensitive(GTK_WIDGET(data->entry_reso_x), 1);
   gtk_widget_set_sensitive(GTK_WIDGET(data->entry_reso_y), 1);
   gtk_widget_set_sensitive(GTK_WIDGET(data->label_reso_times), 1);
}

static void add_if_writable (GdkPixbufFormat *data, GSList **list)
{
   if (gdk_pixbuf_format_is_writable (data))
      *list = g_slist_prepend (*list, data);
}

void export_as_clicked(GtkMenuItem *menuitem, my_plot_callback_data_t *data) {
   GladeXML *gladexml = glade_xml_new(data->gladefile, "filechooserdialog_export_as", NULL);
   GtkWidget *dialog = glade_xml_get_widget(gladexml, "filechooserdialog_export_as");

   eac_callback_data_t eaccbd = {
      .entry_reso_x = GTK_ENTRY(glade_xml_get_widget(gladexml, "entry_reso_x")),
      .entry_reso_y = GTK_ENTRY(glade_xml_get_widget(gladexml, "entry_reso_y")),
      .label_reso_times = glade_xml_get_widget(gladexml, "label_reso_times"),

      .radiobutton_reso_plot = GTK_RADIO_BUTTON(glade_xml_get_widget(gladexml, "radiobutton_reso_plot")),

      .plot = data->plot,
   };

   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_640x480"), "toggled", G_CALLBACK(eac_reso_640x480), &eaccbd);
   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_800x600"), "toggled", G_CALLBACK(eac_reso_800x600), &eaccbd);
   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_1024x768"), "toggled", G_CALLBACK(eac_reso_1024x768), &eaccbd);
   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_1280x1024"), "toggled", G_CALLBACK(eac_reso_1280x1024), &eaccbd);
   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_1600x1200"), "toggled", G_CALLBACK(eac_reso_1600x1200), &eaccbd);
   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_plot"), "toggled", G_CALLBACK(eac_reso_plot), &eaccbd);
   g_signal_connect(glade_xml_get_widget(gladexml, "radiobutton_reso_custom"), "toggled", G_CALLBACK(eac_reso_custom), &eaccbd);

   gulong ce_handler = g_signal_connect(data->plot->da, "configure-event", G_CALLBACK(eac_plot_configure_event), &eaccbd);

   hide_markers(data->plot);

   GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);

#if GTK_CHECK_VERSION(2,6,0)
   // TODO problem in 2.4, it segfaults when calling gtk_file_chooser_add_filter().. :P
   GtkFileFilter *filter = gtk_file_filter_new();
   gtk_file_filter_set_name(filter, "Image formats");
   gtk_file_filter_add_pixbuf_formats(filter);
   gtk_file_chooser_add_filter(chooser, filter);

   filter = gtk_file_filter_new();
   gtk_file_filter_set_name(filter, "All files");
   gtk_file_filter_add_pattern(filter, "*");
   gtk_file_chooser_add_filter(chooser, filter);
#endif

#if GTK_CHECK_VERSION(2,8,0)
   // if we don't have 2.8, it will not ask whether to overwrite :/
   gtk_file_chooser_set_do_overwrite_confirmation(chooser, 1);
#endif

   GSList *image_formats = gdk_pixbuf_get_formats();
   GSList *writable_formats = NULL;
   g_slist_foreach(image_formats, (GFunc)add_if_writable, &writable_formats);
   g_slist_free(image_formats);

   GtkComboBox *combo = GTK_COMBO_BOX(glade_xml_get_widget(gladexml, "combobox_formats"));
   for(GSList *format=writable_formats; format; format=format->next) {
      gtk_combo_box_append_text(combo, gdk_pixbuf_format_get_name(format->data));
   }
   gtk_combo_box_set_active(combo, 0);

   GtkFontButton *fontbutton = GTK_FONT_BUTTON(glade_xml_get_widget(gladexml, "fontbutton"));
   gtk_font_button_set_font_name(fontbutton, data->plot->font);

   GtkToggleButton *bg_white_togglebutton = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "radiobutton_bg_white"));
   gtk_toggle_button_set_active(bg_white_togglebutton, data->plot->draw.inverse_colors);

   do {
      if(gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_ACCEPT) break;

      char *filename = gtk_file_chooser_get_filename(chooser);
      int w = atoi(gtk_entry_get_text(eaccbd.entry_reso_x));
      int h = atoi(gtk_entry_get_text(eaccbd.entry_reso_y));
      int bg_white = gtk_toggle_button_get_active(bg_white_togglebutton);
      if(w < 1 || h < 1) {
	 fprintf(stderr, "Invalid size %dx%d\n", w, h);

      } else {
	 guint format_idx = gtk_combo_box_get_active(combo);
	 GSList *format;
	 char *ext = NULL;
	 if(format_idx <= 0) {
	    ext = strrchr(filename, '.');
	    if(!ext) {
	       GtkWidget *msg = gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "No file extension given");
	       gtk_dialog_run(GTK_DIALOG (msg));
	       gtk_widget_destroy(msg);
	       continue;
	    }
	    ext++;
	    for(format=writable_formats; format; format=format->next) {
	       GdkPixbufFormat *f = format->data;
	       char **exts = gdk_pixbuf_format_get_extensions(f);
	       int i;
	       for(i=0; exts[i]; ++i) {
		  if(!strcasecmp(exts[i], ext)) {
		     printf("Saving image in %s format\n", gdk_pixbuf_format_get_name(f));
		     break;
		  }
	       }
	       char *p = exts[i];
	       g_strfreev(exts);
	       if(p) break;
	    }

	 } else {
	    format = g_slist_nth(writable_formats, format_idx-1);
	    if(!format) {
	       fprintf(stderr, "Internal error - format_idx = %d\n", format_idx);
	    }
	 }

	 if(!format) {
	    GtkWidget *msg = gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "Unknown file extension '%s'", ext);
	    gtk_dialog_run(GTK_DIALOG (msg));
	    gtk_widget_destroy(msg);
	 } else {
	    char *font = gtk_font_button_get_font_name(fontbutton);
	    // font ownership is transferred to export_image, so no need to free it
	    if(!export_image(data->plot, filename, gdk_pixbuf_format_get_name(format->data), w, h, bg_white, font)) {
	       g_free(filename);
	       break;
	    }
	 }
      }
      g_free (filename);
   } while(1);

   g_slist_free(writable_formats);
   g_signal_handler_disconnect(data->plot->da, ce_handler);
   gtk_widget_destroy (dialog);
}

static gboolean export_image(my_plot_t *origplot, const char *filename, const char *format, int width, int height, int backround_white, char *font) {
   my_plot_t *plot = plot_clone(origplot);
   plot->draw.show_disabled = 0;
   plot->draw.inverse_colors = backround_white;
   plot->draw.samples_per_pixel = 1000000000.0;
   plot->scale_font_by_size = 0;

   plot->drawable = gdk_pixmap_new(origplot->da->window, width, height, -1);
   plot->font = font;

   plot->draw.paintapi = paintapi_gtk_new(plot->drawable, plot);

   GdkGC *gc = x_gdk_gc_new_with_fg_color(plot, "black");
   x_gdk_draw_rectangle(plot->drawable, gc, 1, 0, 0, width, height);

   plot_size_or_font_changed(plot);

   plot_draw(&plot->draw);

   GdkPixbuf *pixbuf = gdk_pixbuf_get_from_drawable(NULL, plot->drawable, NULL, 0, 0, 0, 0, -1, -1);
   g_object_unref(plot->drawable);

   const char *key = NULL, *value = NULL;
   if(!strcmp(format, "jpeg")) {
      key = "quality";
      value = "85";
   } else if(!strcmp(format, "png")) {
#if GTK_CHECK_VERSION(2,8,0)
      key = "compression";
      value = "9";
#endif
   }

   int ret = 0;
   GError *error = NULL;
   if(!gdk_pixbuf_save(pixbuf, filename, format, &error, key, value, NULL)) {
      GtkWidget *msg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "Unable to export image: %s\n", error ? error->message : "see console");
      gtk_dialog_run(GTK_DIALOG (msg));
      gtk_widget_destroy(msg);
      ret = 1;
   }
   g_object_unref(pixbuf);

   plot_free(plot);

   return ret;
}
