/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * main - main function setting up some gtk stuff and loading data
 * files given on command line
 */

#include <gtk/gtkmain.h>
#include <glade/glade-xml.h>

#include "main.xh"
#include "plot2.xh"

boolean_t quit = 0;

boolean_t system_check_events(void) {
   while(gtk_events_pending()) {
      gtk_main_iteration();
   }

   if(quit) {
      if(gtk_main_level()) gtk_main_quit();
   }

   return quit;
}

gboolean request_quit(void) {
   if(gtk_main_level()) gtk_main_quit();
   quit = 1;
   return 1;
}

#define setcolor(x,r_,g_,b_) x.r=r_;x.g=g_;x.b=b_;

int main(int arg_count, char **arg) {
   gtk_init(&arg_count, &arg);

   const char *gladefile = "plot.glade";

   GladeXML *gladexml = glade_xml_new(gladefile, "window1", NULL);
   //glade_xml_signal_autoconnect(gladexml);

   GtkWidget *win = glade_xml_get_widget(gladexml, "window1");

   g_signal_connect(win, "destroy-event", G_CALLBACK(request_quit), NULL);
   g_signal_connect(win, "delete-event", G_CALLBACK(request_quit), NULL);

   my_plot_t plot;
   plot_init(&plot);

   plot.parent = GTK_WINDOW(win);
   plot.da = glade_xml_get_widget(gladexml, "drawingarea1");
   plot.draw.show_disabled = 1;
   plot.font = g_strdup("Verdana 17");
   plot.fontsize_at_width = 1600;
   plot.scale_font_by_size = 1;
   plot.draw.show_legend = LEGEND_SHOW_LEFT;

   plot.button_zoom_prev = glade_xml_get_widget(gladexml, "button_zoom_prev");
   plot.button_zoom_next = glade_xml_get_widget(gladexml, "button_zoom_next");
   gtk_widget_set_sensitive(plot.button_zoom_prev, 0);
   gtk_widget_set_sensitive(plot.button_zoom_next, 0);

   setcolor(plot.draw.minor_color, 0x6060, 0x6060, 0x6060);
   setcolor(plot.draw.major_color, 0xc0c0, 0xc0c0, 0xc0c0);
   setcolor(plot.draw.zerogrid_color, 0xffff, 0x6060, 0x4040);
   setcolor(plot.draw.sep_color, 0x8080, 0x8080, 0x8080);
   setcolor(plot.draw.label_color, 0xffff, 0xffff, 0xffff);
   setcolor(plot.draw.cursor_marker_color, 0x8080, 0x8080, 0x8080);
   setcolor(plot.draw.legend_bg_color, 0x2020, 0x2020, 0x2020);
   setcolor(plot.draw.legend_border_color, 0xa0a0, 0xa0a0, 0xa0a0);
   setcolor(plot.draw.legend_text_color, 0xe0e0, 0xe0e0, 0xe0e0);

   /*
     gchar bits[] = { 0 };
     GdkColor color = { 0, 0, 0, 0 };
     GdkCursor *cursor;

     GdkPixmap *pixmap = gdk_bitmap_create_from_data(NULL, bits, 1, 1);
     GdkCursor *cursor = gdk_cursor_new_from_pixmap(pixmap, pixmap, &color, &color, 0, 0);
     gdk_window_set_cursor(GDK_WINDOW(window), cursor);
   */

   GdkCursor *cursor = gdk_cursor_new(GDK_DRAFT_SMALL);
   gdk_window_set_cursor(plot.da->window, cursor);
   gdk_cursor_unref(cursor);

   plot.dsc_x = data_scrollbar_controller_new(gladexml, "xscroll_");
   plot.dsc_y = data_scrollbar_controller_new(gladexml, "yscroll_");

   plot_connect_signals(&plot, gladefile, gladexml);

   plot.progressbar = GTK_PROGRESS_BAR(glade_xml_get_widget(gladexml, "progressbar"));
   plot.menuitem_file_add_data_file = glade_xml_get_widget(gladexml, "menuitem_file_add_data_file");

   if(system_check_events()) return 0;

   for(int a=1; a<arg_count; ++a) {
      add_datafile(&plot, arg[a]);
   }

   // set initial center
   double min, max;
   data_scrollbar_controller_get_range(plot.dsc_y, &min, &max);
   data_scrollbar_controller_set_zoompoint(plot.dsc_y, min > 0 ? 0 : max < 0 ? 1 : -min / (max - min)); // align to(wards) zero
   data_scrollbar_controller_set_zoompoint(plot.dsc_x, 1.0); // align to right

   if(!quit) gtk_main();

   return 0;
}
