/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * stringutil - misc functions for operating C strings
 */

//#define USE_STDLIB

#include <ctype.h>
#ifndef USE_STDLIB
#include <math.h>
#else
#include <stdlib.h>
#endif

#include "stringutil.xh"


char *strnzdup(const char *p, int maxlen) {

   int len = strlen(p);
   if(len > maxlen) len = maxlen;
   char *new = (char *)malloc(len + 1);
   if(!new) return NULL;
   memcpy(new, p, len);
   new[len] = '\0';
   return new;
}

int chomp(char *p) {
   p += strlen(p) - 1;
   if(*p != '\n') return 0;
   *p = 0;
   return 1;
}

const char *skipws(const char *p) {
   while(isspace(*p)) p++;
   return p;
}

const char *skipnonws(const char *p) {
   while(*p && !isspace(*p)) p++;
   return p;
}

char *strip_comments(char *line) {
  char *comment = strchr(line, '#');
  if(comment) *comment = '\0';
  return line;
}

static inline int NUMBER(int c) { return c >= 0x30 && c <= 0x39; }

#deftemplate parse_float_impl(TYPE)
TYPE parse_TYPE_num(const char **p) {
#ifndef USE_STDLIB
#if 1 // xkr47 v1
   TYPE data = 0;
   const char *q = *p;
   int neg;
   if(*q == '-') {
      neg = 1;
      q++;
   } else {
      if(*q == '+') q++;      
      neg = 0;
   }
  
   for(; NUMBER(*q); ++q) {
      data *= 10;
      data += *q - '0';
   }

   if(*q == '.' || *q == ',') {
      int gotdot = 10;
      for(q++; NUMBER(*q); ++q) {
	 data += ((double)(*q - '0')) / gotdot;
	 gotdot *= 10;
      }
   }
  
   *p = q;
   if(neg) data = -data;
   return data;
#elif 1 // mokki v1
   TYPE data;
   long d = 0;
   const char *q = *p;
   int neg = 1;
   if(*q == '-') {
      neg = -1;
      q++;
   }
   for(; NUMBER(*q); ++q) {
      d *= 10;
      d += *q - '0';
   }
   if(*q == '.' || *q == ',') {
      int decimals = 0;
      long frac = 0;
      decimals = 0;
      for(++q; NUMBER(*q); ++q) {
	 frac *= 10;
	 frac += *q - '0';
	 decimals++;
      }
      data = neg*(d + frac/exp10((double)decimals));
   } else {
      data = neg*d;
   }
   *p = q;
   return data;
#endif
#else
   return (TYPE)strtod(*p, p);
#endif
}
#endtemplate

#template parse_float_impl(double)

#deftemplate parse_integer_impl(TYPE)
TYPE parse_TYPE_num(const char **p) {
#ifndef USE_STDLIB
   TYPE data = 0;
   const char *q = *p;
   int neg;
   if(*q == '-') {
      neg = 1;
      q++;
   } else {
      neg = 0;
   }

   for(; NUMBER(*q); ++q) {
      data *= 10;
      data += *q - '0';
   }
   *p = q;
   if(neg) data = -data;
   return data;
#else
   return (TYPE)strtol(*p, p, 10);
#endif
}
#endtemplate

#template parse_integer_impl(int)

